% (c) 2009 Lehrstuhl fuer Softwaretechnik und Programmiersprachen, 
% Heinrich Heine Universitaet Duesseldorf
% This software is licenced under EPL 1.0 (http://www.eclipse.org/org/documents/epl-v10.html) 

/* 
 ensure_loaded('images/state_viewer_images').
*/

/* ------------------------------------------------------------------ */

:- use_module('../bmachine',[b_machine_name/1, b_get_definition_from_machine/2,
                b_get_definition_string_from_machine/2]).
:- use_module('../itracer',[state_corresponds_to_initialised_b_machine/2]).
:- use_module('../kernel_objects',[expand_closure/2]).

/* First argument: image list of pairs: ImageName, ImageFilePath
   Second argument: list of triplets {type framename label}
   */
                
                
tcltk_get_image_list(ImageList,FramesAndItems) :-
       current_expression(_CurID,State,Templ), call(Templ),
       itracer:state_corresponds_to_initialised_b_machine(State,BState),
       b_get_definition_from_machine('ANIMATION_FUNCTION',_),
       get_animation_images(ImageList),
       %print(animation_images(ImageList)),
       eval_definition_fun('ANIMATION_FUNCTION', BState, FunctionRes1),
       (eval_definition_fun('ANIMATION_FUNCTION_DEFAULT', BState, FunctionRes0)
         -> mnf(user:override_anim_fun(FunctionRes0,FunctionRes1,FunctionRes))
          ; FunctionRes = FunctionRes1
       ),
       print(full_fun(FunctionRes)),nl,
       (FunctionRes = int(ImageNr)
         -> (/* simple style animation: just a single image */
             convert_nr_into_tk_label(ImageNr,"IMG",Label),
             FramesAndItems = [frame, row1, top, img11, row1, Label]
             )
          ;  ( get_min_max_row(FunctionRes,Min,Max)
               -> (print('Matrix function'),nl,
                   print(min_max(Min,Max)),nl,
                   setup_frames(Min,Max,Frames), print(frames(Frames)),nl,
                   sort(FunctionRes,SortedFR),
                   get_frame_items(SortedFR,Items),
                   print(items(Items)),nl,
                   append(Frames,Items,FramesAndItems)
                  )
               ;  (print('**** Unknown Format for ANIMATION_FUNCTION result'),nl,
                   print('**** must be of type INT or (INT*INT)+->INT'),nl,
                   fail
                  )
             )
       ).


eval_definition_fun(ANIMATION_FUNCTION_STR,BState, FunctionRes) :-
       b_get_definition_from_machine(ANIMATION_FUNCTION_STR,AnimFunction),
       b_interpreter:b_compute_expression(AnimFunction,[],BState,FunctionResCl),
       mnf( expand_closure(FunctionResCl, ExpCl) ),
       (translate_to_ints(ExpCl, FunctionRes) -> true /* already translate to ints so that override will work properly */
          ; (add_error(state_viewer_images,'Invalid animation function: ',(ANIMATION_FUNCTION_STR,ExpCl)),
             fail)
       )   ,print(fres(FunctionRes)),nl
       .
        
translate_to_ints([],[]).
translate_to_ints(int(X),int(X)).
translate_to_ints([((Row,Col),Img)|T], [((RI,CI),II)|TT]) :-
   get_int(Row,RI), get_int(Col,CI), get_int(Img,II),
   translate_to_ints(T,TT).

/* a version of override which does not worry about correct typing */
override_anim_fun([],Fun2,Res) :- !, Res=Fun2.
override_anim_fun(int(_),Fun2,Res) :- !, Res=Fun2.
override_anim_fun([((RowT,Col),Img)|T],Fun2,Res) :- !,
   (member(((RowT,Col),_),Fun2)
     -> Res = ResT
      ; Res = [((RowT,Col),Img)|ResT]
   ), override_anim_fun(T,Fun2,ResT).
override_anim_fun(X,F,R) :-
   add_error(state_viewer_images,'Unknown Format for ANINMATION_FUNCTION_DEFAULT', X),
   R=F.

get_min_max_row([((Row,_Col),_Img)|T],Min,Max) :-
   =(Row,RowInt),
   get_min_max_row2(T,RowInt,RowInt,Min,Max).
get_min_max_row2([],Min,Max,Min,Max).
get_min_max_row2([((RowT,_Col),_Img)|T],Min,Max,MinR,MaxR) :-
    =(RowT,Row),
    (Row<Min -> Min1=Row ; Min1=Min),
    (Row>Max -> Max1=Row ; Max1=Max),
    get_min_max_row2(T,Min1,Max1,MinR,MaxR).
    
get_int(int(R),R).
get_int(fd(R,_Name),R).
get_int(term(bool(X)),X).

get_frame_items([],[]).
get_frame_items([((RowT,ColT),ImgT)|T],[ItemLbl,FrameLbl,ImageLbl|TR]) :-
    =(RowT,Row), =(ColT,Col),
    =(ImgT,Img),
    convert_nr_into_tk_label(Row,"frame",FrameLbl),
    (images(Img,_,ImageLbl) 
      -> convert_2nr_into_tk_label(Row,Col,"item",ItemLbl)
      ;  (ItemLbl = text,
          (get_animation_string(Img,ImageLbl) -> true
            ; convert_nr_into_tk_label(Img,"ERR",ImageLbl)
         ))
    ),
    get_frame_items(T,TR).

setup_frames(Min,Max,[]) :- Min>Max,!.
setup_frames(Min,Max,[frame, FrameLbl, top |T]) :-
    convert_nr_into_tk_label(Min,"frame",FrameLbl), M1 is Min+1,
    setup_frames(M1,Max,T).

:- dynamic images/3.

get_animation_images(Res) :-
    findall( img(Nr,S,Lbl), get_animation_image(Nr,S,Lbl), List),
    retractall(images(_,_,_)),
    flatten_img_list(List,Res).

flatten_img_list([],[]).
flatten_img_list([img(Nr,S,Lbl)|T],[Lbl,S|FT]) :- assert(images(Nr,S,Lbl)), flatten_img_list(T,FT).

get_animation_string(Nr,S) :-
         number_codes(Nr,TailCodes),
         /* ANIMATION_STR */
         atom_codes(Def_Name,[65,78,73,77,65,84,73,79,78,95,83,84,82|TailCodes]),
         b_get_definition_string_from_machine(Def_Name,S).
         
get_animation_image(Nr,S,Lbl) :-
         b_get_definition_string_from_machine(Def_Name,S),
         /* ANIMATION_IMG */
         atom_codes(Def_Name,[65,78,73,77,65,84,73,79,78,95,73,77,71|TailCodes]),
         number_codes(Nr,TailCodes),
         convert_nr_into_tk_label(Nr,"IMG",Lbl).
                
convert_nr_into_tk_label(Nr,Prefix,Label) :- number_codes(Nr,Codes),
        append(Prefix,Codes,C),
        atom_codes(Label,C).
        
convert_2nr_into_tk_label(Nr1,Nr2,Prefix,Label) :- number_codes(Nr1,Codes1), number_codes(Nr2,Codes2),
        append("_",Codes2,C2), append(Codes1,C2,Codes),
        append(Prefix,Codes,C),
        atom_codes(Label,C).
 